﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Builder
{
    public partial class MainWindow : Form
    {
        public MainWindow ()
        {
            InitializeComponent ();
            createColorPanels (new List<String> { "red", "green", "blue" });
            createComponent (typeof (Record));
            createComponent (typeof (Field));
            createComponent (typeof (Button));
            createComponent (typeof (Panel));
            createComponent (typeof (TreeView));
            createComponent (typeof (ListView));
            createComponent (typeof (TextBox));
            createComponent (typeof (DataGrid));
            designer.AllowDrop = true;
            treeView.AllowDrop = true;
        }

        private void createColorPanels (List<String> colorNames)
        {
            foreach (var colorName in colorNames)
            {
                Color color = Color.FromName (colorName);
                ColorPanel p = new ColorPanel (color, colorName);
                int n = colorPage.Controls.Count;
                p.Left = n * p.Width + (n + 1) * p.Left;
                colorPage.Controls.Add (p);
            }
        }

        private void createComponent (Type type)
        {
            Factory f = new Factory (type);

            int n = componentPage.Controls.Count;
            f.Left = f.Left + n * (f.Width + f.Left);
            componentPage.Controls.Add (f);
        }

        private void designer_DragEnter (object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent (typeof (Color)))
                e.Effect = DragDropEffects.Move;
            // else if (e.Data.GetDataPresent (DataFormats.Text))
            //    e.Effect = DragDropEffects.Copy;
            // else if (e.Data.GetDataPresent ("System.Windows.Forms.Button"))
            //     e.Effect = DragDropEffects.Copy;
            else if (e.Data.GetDataPresent ("Builder.Factory"))
                e.Effect = DragDropEffects.Copy;
            else
                e.Effect = DragDropEffects.None;
        }

        private void designer_DragDrop (object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent ("Builder.Factory"))
            {
                Factory factory = e.Data.GetData ("Builder.Factory") as Factory;
                Object obj = factory.createInstance ();
                if (obj is Control)
                {
                    Control ctl = obj as Control;
                    Point location = designer.PointToClient (new Point (e.X, e.Y));
                    ctl.Left = location.X;
                    ctl.Top = location.Y;
                    designer.Controls.Add (ctl);
                }
            }
        }

        private void treeView_DragEnter (object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent ("Builder.Factory"))
                e.Effect = DragDropEffects.Copy;
        }

        private void treeView_DragDrop (object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent ("Builder.Factory"))
            {
                Factory factory = e.Data.GetData ("Builder.Factory") as Factory;
                Object obj = factory.createInstance ();

                MyNode node = new MyNode ();
                node.Text = obj.ToString ();
                node.Tag = obj;
                node.connect ();

                Point pt = treeView.PointToClient (new Point (e.X, e.Y));
                TreeNode target = treeView.GetNodeAt (pt);
                if (target == null)
                    treeView.Nodes.Add (node);
                else
                    target.Nodes.Add (node);
            }
        }

        private void treeView_NodeMouseClick (object sender, TreeNodeMouseClickEventArgs e)
        {
            Object obj = e.Node.Tag;
            propGrid.SelectedObject = new ObjectWrapper (obj);
        }
    }
}
